@@ -1,6 +1,7 @@
 # -*- coding: utf-8 -*-
 # synadm
-# Copyright (C) 2020-2023 Johannes Tiefenbacher
+# Copyright (C) 2020-2025 Johannes Tiefenbacher
+# Copyright (C) 2023-2025 JacksonChen666
 #
 # synadm is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -28,12 +29,45 @@ documentation of the Synapse Admin APIs and the Matrix spec at
 https://matrix.org/docs/spec/#matrix-apis.
 """
 
-import requests
-from http.client import HTTPConnection
 import datetime
 import json
-import urllib.parse
 import re
+import traceback
+import urllib.parse
+from http.client import HTTPConnection
+from typing import Any, Dict, List, Optional, Union
+import importlib.metadata
+
+import requests
+import requests_unixsocket
+
+
+def log_fatal_exit(error, logger, message=None):
+    """Log a fatal error and exit synadm.
+
+    Args:
+        error: A Python exception.
+        logger: A Python logger, with info and fatal methods
+        message: Message to use instead of "synadm exited due to a fatal
+            error."
+    """
+    if message is None:
+        message = "synadm exited due to a fatal error."
+
+    # format_exception() returns a list of strings (with new lines). join it
+    # into a single string again for readability.
+    #
+    # error specified multiple times to ensure it works with python 3.9 and
+    # versions after that which accept things differently
+    logger.info("".join(traceback.format_exception(error, error,
+                                                   error.__traceback__)))
+
+    logger.fatal(
+        "%s: %s.\n\n%s",
+        type(error).__name__,
+        error, message
+    )
+    raise SystemExit(1) from error
 
 
 class ApiRequest:
@@ -41,8 +75,8 @@ class ApiRequest:
 
     This is subclassed by SynapseAdmin and Matrix
     """
-    def __init__(self, log, user, token, base_url, path, timeout, debug,
-                 verify=None):
+    def __init__(self, log, user, token, protocol, base_url, path, timeout,
+                 debug, verify=None):
         """Initialize an APIRequest object
 
         Args:
@@ -51,7 +85,10 @@ class ApiRequest:
                 This can either be the fully qualified Matrix user ID, or just
                 the localpart of the user ID.
             token (string): the API user's token
-            base_url (string): URI e.g https://fqdn:port
+            protocol (string): Either the value "http" (supports HTTPS) or
+                "unix" (local path to unix socket in base_url).
+            base_url (string): URI e.g https://fqdn:port if protocol is
+                "http", path to local unix socket if protocol is "unix".
             path (string): the path to the API endpoint; it's put after
                 base_url to form the basis for all API endpoint paths
             timeout (int): requests module timeout used in query method
@@ -62,57 +99,125 @@ class ApiRequest:
         self.log = log
         self.user = user
         self.token = token
-        self.base_url = base_url.strip("/")
+        self.protocol = protocol
+        # .strip removes / on prefix side, breaking unix socket paths,
+        # remove suffix once instead
+        self.base_url = base_url.removesuffix("/")
         self.path = path.strip("/")
+
+        version = ""
+        try:
+            version = "/" + importlib.metadata.version("synadm")
+        except importlib.metadata.PackageNotFoundError:
+            self.log.debug("Could not fetch package version for user agent")
         self.headers = {
             "Accept": "application/json",
-            "Authorization": "Bearer " + self.token
+            "Authorization": "Bearer " + self.token,
+            "User-Agent": f"synadm{version}"
         }
+
         self.timeout = timeout
         if debug:
             HTTPConnection.debuglevel = 1
         self.verify = verify
 
-    def query(self, method, urlpart, params=None, data=None, token=None,
-              base_url_override=None, verify=None, *args, **kwargs):
+    def query(
+        self,
+        method: str,
+        urlpart: str,
+        *args: Any,
+        params: Optional[Dict[str, Any]] = None,
+        data: Optional[Dict[str, Any]] = None,
+        token: Optional[str] = None,
+        protocol_override: Optional[str] = None,
+        base_url_override: Optional[bool] = False,
+        verify: Optional[bool] = None,
+        **kwargs: Dict[str, Any],
+    ) -> Optional[Union[Dict[str, Any], List[Dict[str, Any]]]]:
         """Generic wrapper around requests methods.
 
         Handles requests methods, logging and exceptions, and URL encoding.
 
+        The arguments "method" and "params" are passed to requests.request
+        as is.
+
         Args:
-            urlpart (string): The path to the API endpoint, excluding
-                self.base_url and self.path (the part after
-                proto://fqdn:port/path). It will be passed to Python's
-                str.format, so the string should not be already formatted
-                (as f-strings or with str.format) as to sanitize the URL.
-            params (dict, optional): URL parameters (?param1&paarm2).  Defaults
-                to None.
-            data (dict, optional): Request body used in POST, PUT, DELETE
-                requests.  Defaults to None.
-            base_url_override (bool): The default setting of self.base_url set
+            method: The http method to use (get, post, put, ...)
+            urlpart: The path to the API endpoint, excluding self.base_url and
+                self.path (the part after proto://fqdn:port/path). It will be
+                passed to Python's str.format, so the string should not be
+                already formatted (as f-strings or with str.format) as to
+                sanitize the URL. Values will be used from **kwargs.
+            params (dict): URL queries (?param1&param2). Use a list type
+                for the value to achieve ?param1=value1&param1=value2.
+            data: Request body used in POST, PUT, DELETE requests, encoded
+                as JSON.
+            token: An optional token overriding the configured one.
+            protocol_override: Can either be "http" or "unix", and will
+                communicate to Synapse via the specified thing. Necessary
+                when using base_url_override, default None to make noise.
+            base_url_override: The default setting of self.base_url set
                 on initialization can be overwritten using this argument.
-            verify(bool): Mandatory SSL verification is turned on by default
-                and can be turned off using this method.
+            verify: Mandatory SSL verification is on by default and can be
+                turned off using this method.
             *args: Arguments that will be URL encoded and passed to Python's
                 str.format.
             **kwargs: Keyword arguments that will be URL encoded (only the
                 values) and passed to Python's str.format.
 
         Returns:
-            string or None: Usually a JSON string containing
-                the response of the API; responses that are not 200(OK) (usally
-                error messages returned by the API) will also be returned as
-                JSON strings. On exceptions the error type and description are
-                logged and None is returned.
+            A dict, list or None. If it's a dict or list, it is a JSON
+            decoded response. Whether it is a dict or a list depends on what
+            the server responds with. It returns a None if any exceptions
+            are encountered.
         """
-        args = list(args)
-        kwargs = dict(kwargs)
-        for i in range(len(args)):
-            args[i] = urllib.parse.quote(args[i], safe="")
-        for i in kwargs.keys():
-            kwargs[i] = urllib.parse.quote(kwargs[i], safe="")
+        args = [urllib.parse.quote(arg, safe="") for arg in args]
+        kwargs = {k: urllib.parse.quote(v, safe="") for k, v in kwargs.items()}
         urlpart = urlpart.format(*args, **kwargs)
 
+        protocol = self.protocol
+        # if base_url is overridden, we can't assume protocol
+        #
+        # so also take the override protocol, which can be none if not
+        # specified
+        if base_url_override:
+            protocol = protocol_override
+
+        _requests_library = None
+        if protocol == "unix":
+            _requests_library = requests_unixsocket
+            # because the "host" includes / which is ambiguous with path,
+            # URL encode the "host" (path to unix socket).
+            # requests_unixsocket does it in the example anyways
+            #
+            # also, the `http://` part is provided by the user in the case
+            # of http protocol, but not this case, so include that necessary
+            # prefix
+            self.base_url = "http+unix://" + urllib.parse.quote(
+                                                self.base_url, safe="")
+
+            # i don't like this, and this is being applied to all unix
+            # socket requests instead of specific thigns, but pretending to
+            # be the reverse proxy that is proxying from ::1 (ipv6
+            # localhost, arbitrary but isn't wrong) is necessary so that
+            # synapse doesn't scream in confusion because suddenly the login
+            # request (rate limited by IP) is coming from an unspecified IP
+            # (hello unix sockets, i guess)
+            self.headers["X-Forwarded-For"] = "::1"
+            self.headers["X-Forwarded-Proto"] = "https"
+        elif protocol == "http":
+            _requests_library = requests
+        else:
+            self.log.error("query(): Somehow protocol is set to neither "
+                           f"unix or http, but instead '{protocol}'. "
+                           "Manually changed setting?")
+
+            # this outputs stuff like
+            # <synadm.api.MiscRequest object at 0x7f7f11a4b770>
+            self.log.debug(self)
+
+            # let it go try .request on None to error out
+
         if base_url_override:
             self.log.debug("base_url override!")
             url = f"{base_url_override}/{self.path}/{urlpart}"
@@ -120,7 +225,8 @@ class ApiRequest:
         else:
             url = f"{self.base_url}/{self.path}/{urlpart}"
             host_descr = "Synapse"
-        self.log.info("Querying %s on %s", method, url)
+        self.log.info("Querying %s on %s with URL query %s", method, url,
+                      params)
 
         if token:
             self.log.debug("Token override! Adjusting headers.")
@@ -131,7 +237,7 @@ class ApiRequest:
             override_verify = verify
 
         try:
-            resp = requests.request(
+            resp = _requests_library.request(
                 method, url, headers=self.headers, timeout=self.timeout,
                 params=params, json=data, verify=override_verify
             )
@@ -139,6 +245,14 @@ class ApiRequest:
                 self.log.warning(f"{host_descr} returned status code "
                                  f"{resp.status_code}")
             return resp.json()
+        except requests.exceptions.SSLError as error:
+            log_fatal_exit(error, self.log, "TLS/SSL error. If you have a "
+                           "Synapse with a self-signed certificate, you "
+                           "can configure synadm to ignore certificate "
+                           "errors.")
+        except requests.exceptions.ConnectionError as error:
+            log_fatal_exit(error, self.log, "Connection error. Please check "
+                           "that Synapse can be reached.")
         except Exception as error:
             self.log.error("%s while querying %s: %s",
                            type(error).__name__, host_descr, error)
@@ -231,16 +345,20 @@ class MiscRequest(ApiRequest):
         """
         super().__init__(
             log, "", "",  # Set user and token to empty string
-            "", "",  # Set base_url and path to empty string
+            "", "", "",  # Set protocol, base_url and path to empty string
             timeout, debug, verify
         )
 
-    def federation_uri_well_known(self, base_url):
+    def federation_uri_well_known(self, protocol, base_url):
         """Retrieve the URI to the Server-Server (Federation) API port via the
         .well-known resource of a Matrix server/domain.
 
         Args:
-            base_url: proto://name or proto://fqdn
+            protocol (string): Either "http" or "unix".
+            base_url (string): proto://name:port or
+                proto://fqdn:port with "http" protocol, or an absolute path
+                with "unix" protocol.
+
 
         Returns:
             string: https://fqdn:port of the delegated server for Server-Server
@@ -248,7 +366,7 @@ class MiscRequest(ApiRequest):
         """
         resp = self.query(
             "get", ".well-known/matrix/server",
-            base_url_override=base_url,
+            protocol_override=protocol, base_url_override=base_url,
         )
         if resp is not None:
             if ":" in resp["m.server"]:
@@ -266,7 +384,7 @@ class Matrix(ApiRequest):
         ApiRequest (object): parent class containing general properties and
             methods for requesting REST API's
     """
-    def __init__(self, log, user, token, base_url, matrix_path,
+    def __init__(self, log, user, token, protocol, base_url, matrix_path,
                  timeout, debug, verify):
         """Initialize the Matrix API object
 
@@ -276,7 +394,10 @@ class Matrix(ApiRequest):
                 unused); This can either be the fully qualified Matrix user ID,
                 or just the localpart of the user ID.
             token (string): the Matrix API user's token
-            base_url (string): URI e.g https://fqdn:port
+            protocol: Either "http" or "unix". Affects how the base_url
+                option is interpreted.
+            base_url (string): URI e.g https://fqdn:port ("http" protocol),
+                or an absolute path when protocol is set to "unix"
             path (string): the path to the API endpoint; it's put after
                 base_url and forms the basis for all API endpoint paths
             timeout (int): requests module timeout used in ApiRequest.query
@@ -287,7 +408,7 @@ class Matrix(ApiRequest):
         """
         super().__init__(
             log, user, token,
-            base_url, matrix_path,
+            protocol, base_url, matrix_path,
             timeout, debug, verify
         )
         self.user = user
@@ -360,19 +481,23 @@ class Matrix(ApiRequest):
 
         return self.query(method, endpoint, data=data_dict, token=token)
 
-    def server_name_keys_api(self, server_server_uri):
+    def server_name_keys_api(self, protocol, server_server_uri):
         """Retrieve the Matrix server's own homeserver name via the
         Server-Server (Federation) API.
 
         Args:
-            server_server_uri (string): proto://name:port or proto://fqdn:port
+            protocol (string): Either "http" or "unix".
+            server_server_uri (string): proto://name:port or
+                proto://fqdn:port with "http" protocol, or an absolute path
+                with "unix" protocol.
 
         Returns:
             string: The Matrix server's homeserver name or FQDN, usually
             something like matrix.DOMAIN or DOMAIN
         """
         resp = self.query(
-            "get", "key/v2/server", base_url_override=server_server_uri
+            "get", "key/v2/server",
+            protocol_override=protocol, base_url_override=server_server_uri
         )
         if not resp or not resp.get("server_name"):
             self.log.error("The homeserver name could not be fetched via the "
@@ -388,8 +513,8 @@ class SynapseAdmin(ApiRequest):
         ApiRequest (object): parent class containing general properties and
             methods for requesting REST API's
     """
-    def __init__(self, log, user, token, base_url, admin_path, timeout, debug,
-                 verify):
+    def __init__(self, log, user, token, protocol, base_url, admin_path,
+                 timeout, debug, verify):
         """Initialize the SynapseAdmin object
 
         Args:
@@ -398,7 +523,10 @@ class SynapseAdmin(ApiRequest):
                 This can either be the fully qualified Matrix user ID,
                 or just the localpart of the user ID. FIXME is that true?
             token (string): The admin user's token
-            base_url (string): URI e.g https://fqdn:port
+            protocol: Either "http" or "unix". Affects how the base_url
+                option is interpreted.
+            base_url (string): URI e.g https://fqdn:port ("http" protocol),
+                or an absolute path when protocol is set to "unix"
             path (string): The path to the API endpoint; it's put after
                 base_url and the basis for all API endpoint paths
             timeout (int): Requests module timeout used in ApiRequest.query
@@ -409,15 +537,18 @@ class SynapseAdmin(ApiRequest):
         """
         super().__init__(
             log, user, token,
-            base_url, admin_path,
+            protocol, base_url, admin_path,
             timeout, debug, verify
         )
         self.user = user
 
     def user_list(self, _from, _limit, _guests, _deactivated,
-                  _name, _user_id, _admin=None):
+                  _name, _user_id, _admin=None, order_by=None, _dir=None,
+                  not_user_type=None, locked=False):
         """List and search users
 
+        https://element-hq.github.io/synapse/latest/admin_api/user_admin_api.html#list-accounts
+
         Args:
             _from (int): offsets user list by this number, used for pagination
             _limit (int): maximum number of users returned, used for pagination
@@ -428,6 +559,19 @@ class SynapseAdmin(ApiRequest):
             _user_id (string): fully qualified Matrix user ID to search for
             _admin (bool or None): whether to filter for admins. a None
                 does not filter.
+            order_by (string or None): Method of sorting returned users. See
+                Synapse's documentation for valid choices. None uses Synapse
+                defaults. Note that methods other than "name" or
+                "creation_ts" may be slow and cause a major load as they
+                have no database index.
+            _dir (string or None): Direction of sorting. "f" for forwards
+                (default or when None), "b" for backwards/reversed.
+            not_user_type (string or list or None): User type to exclude.
+                Supports list for multiple values. Possible values are
+                "bot", "support", or "" (to exclude users without a user
+                type). Defaults to None for no filtering.
+            locked (bool): Include locked users. Defaults to not
+                including them.
 
         Returns:
             string: JSON string containing the found users
@@ -439,14 +583,19 @@ class SynapseAdmin(ApiRequest):
                        else None),
             "deactivated": "true" if _deactivated else None,
             "name": _name,
-            "user_id": _user_id
+            "user_id": _user_id,
+            "order_by": order_by,
+            "dir": _dir,
+            "not_user_type": not_user_type,
+            "locked": str(locked).lower()
         }
         if _admin is not None:
             params["admins"] = str(_admin).lower()
         return self.query("get", "v2/users", params=params)
 
-    def user_list_paginate(self, _limit, _guests, _deactivated,
-                           _name, _user_id, _from="0", admin=None):
+    def user_list_paginate(self, _limit, _guests, _deactivated, _name,
+                           _user_id, _from="0", admin=None, order_by=None,
+                           _dir=None, not_user_type=None, locked=False):
         # documentation is mostly duplicated from user_list...
         """Yields API responses for all of the pagination.
 
@@ -461,6 +610,19 @@ class SynapseAdmin(ApiRequest):
             _user_id (string): Fully qualified Matrix user ID to search for.
             _from (string): Offsets user list by this number, used for
                 pagination.
+            order_by (string or None): Method of sorting returned users. See
+                Synapse's documentation for valid choices. None uses Synapse
+                defaults. Note that methods other than "name" or
+                "creation_ts" may be slow and cause a major load as they
+                have no database index.
+            _dir (string or None): Direction of sorting. "f" for forwards
+                (default or when None), "b" for backwards/reversed.
+            not_user_type (string or list or None): User type to exclude.
+                Supports list for multiple values. Possible values are
+                "bot", "support", or "" (to exclude users without a user
+                type). Defaults to None for no filtering.
+            locked (bool): Include locked users. Defaults to not
+                including them.
 
         Yields:
             dict: The Admin API response for listing accounts.
@@ -468,7 +630,8 @@ class SynapseAdmin(ApiRequest):
         """
         while _from is not None:
             response = self.user_list(_from, _limit, _guests, _deactivated,
-                                      _name, _user_id, admin)
+                                      _name, _user_id, admin, order_by,
+                                      _dir, not_user_type, locked)
             yield response
             _from = response.get("next_token", None)
 
@@ -513,6 +676,49 @@ class SynapseAdmin(ApiRequest):
             "erase": gdpr_erase
         }, user_id=user_id)
 
+    def user_suspend(self, user_id):
+        """
+        Suspend an account to prevent it from doing anything, essentially
+        freezing it.
+
+        To query the status, use the user_details function or the Query User
+        Account API
+        (https://element-hq.github.io/synapse/latest/admin_api/user_admin_api.html#query-user-account).
+
+        https://element-hq.github.io/synapse/latest/admin_api/user_admin_api.html#suspendunsuspend-account
+
+        Args:
+            user_id (string): fully qualified Matrix user ID
+
+        Returns:
+            string: JSON string containing the Admin API's response or None if
+            an exception occured. See Synapse Admin API docs for details.
+        """
+        data = {"suspend": True}
+        return self.query("PUT", "v1/suspend/{user_id}", data=data,
+                          user_id=user_id)
+
+    def user_unsuspend(self, user_id):
+        """
+        Unsuspend an account to prevent it from doing anything.
+
+        To query the status, use the user_details function or the Query User
+        Account API
+        (https://element-hq.github.io/synapse/latest/admin_api/user_admin_api.html#query-user-account).
+
+        https://element-hq.github.io/synapse/latest/admin_api/user_admin_api.html#suspendunsuspend-account
+
+        Args:
+            user_id (string): fully qualified Matrix user ID
+
+        Returns:
+            string: JSON string containing the Admin API's response or None if
+            an exception occured. See Synapse Admin API docs for details.
+        """
+        data = {"suspend": False}
+        return self.query("PUT", "v1/suspend/{user_id}", data=data,
+                          user_id=user_id)
+
     def user_password(self, user_id, password, no_logout):
         """Set the user password, and log the user out if requested
 
@@ -600,7 +806,8 @@ class SynapseAdmin(ApiRequest):
                           user_id=user_id)
 
     def user_modify(self, user_id, password, display_name, threepid,
-                    avatar_url, admin, deactivation, user_type, lock):
+                    avatar_url, admin, deactivation, user_type, lock,
+                    logout_devices):
         """ Create or update information about a given user
 
         The threepid argument must be passed as a tuple in a tuple (which is
@@ -631,6 +838,8 @@ class SynapseAdmin(ApiRequest):
         if user_type:
             data.update({"user_type": None if user_type == 'null' else
                          user_type})
+        if logout_devices:
+            data.update({"logout_devices": logout_devices})
         return self.query("put", "v2/users/{user_id}", data=data,
                           user_id=user_id)
 
@@ -755,6 +964,49 @@ class SynapseAdmin(ApiRequest):
         return self.query("get", "v1/threepid/{medium}/users/{address}",
                           address=address, medium=medium)
 
+    def user_redact(self, user, rooms, reason=None, limit=None):
+        """
+        Redacts all events by a local or remote user in specified or all
+        rooms.
+
+        For users local to the homeserver, Synapse will redact it as the
+        specified user. For remote users, Synapse will redact as anyone who
+        has sufficient power level to remove messages.
+
+        https://element-hq.github.io/synapse/latest/admin_api/user_admin_api.html#redact-all-the-events-of-a-user
+
+        Args:
+            user: An MXID in string
+            rooms: A list of rooms to redact events in. If the list is
+                empty, events in all rooms are redacted.
+            reason: A string, specifying the reason to redact. This is
+                visible to everyone in the room and will be part of the
+                redaction event.
+            limit: An integer of how many events can be redacted at once.
+                Synapse defaults to 1000.
+        """
+        data = {
+            "rooms": rooms or []
+        }
+        if reason:
+            data["reason"] = reason
+        if limit:
+            data["limit"] = limit
+        return self.query("post", "v1/user/{user_id}/redact", data=data,
+                          user_id=user)
+
+    def user_redact_status(self, redact_id):
+        """
+        Check the status of redacting user messages.
+
+        https://element-hq.github.io/synapse/latest/admin_api/user_admin_api.html#check-the-status-of-a-redaction-process
+
+        Args:
+            redact_id: The redaction ID from the user redaction API.
+        """
+        return self.query("get", "v1/user/redact_status/{redact_id}",
+                          redact_id=redact_id)
+
     def room_join(self, room_id_or_alias, user_id):
         """ Allow an administrator to join an user account with a given user_id
         to a room with a given room_id_or_alias
@@ -763,18 +1015,28 @@ class SynapseAdmin(ApiRequest):
         return self.query("post", "v1/join/{room_id_or_alias}", data=data,
                           room_id_or_alias=room_id_or_alias)
 
-    def room_list(self, _from, limit, name, order_by, reverse):
+    def room_list(self, _from, limit, name, order_by, reverse,
+                  empty_rooms=None):
         """ List and search rooms
+
+        args:
+            empty_rooms: Whether to get empty rooms. Default is None, which
+                gets both empty and non-empty rooms. Returns empty rooms if
+                True, and non-empty rooms if False.
         """
-        return self.query("get", "v1/rooms", params={
+        params = {
             "from": _from,
             "limit": limit,
             "search_term": name,
             "order_by": order_by,
             "dir": "b" if reverse else None
-        })
+        }
+        if empty_rooms is not None:
+            params["empty_rooms"] = str(empty_rooms).lower()
+        return self.query("get", "v1/rooms", params=params)
 
-    def room_list_paginate(self, limit, name, order_by, reverse, _from=0):
+    def room_list_paginate(self, limit, name, order_by, reverse, _from=0,
+                           empty_rooms=None):
         """ Yields API responses for room listing.
 
         Args:
@@ -785,19 +1047,26 @@ class SynapseAdmin(ApiRequest):
             order_by (string): Synapse Room list API specific argument.
             reverse (bool): Whether the results should be
             _from (int): Initial offset in pagination.
+            empty_rooms: Whether to get empty rooms. Default is None, which
+                gets both empty and non-empty rooms. Returns empty rooms if
+                True, and non-empty rooms if False.
 
         Yields:
             dict: The Admin API response for listing accounts.
                 https://element-hq.github.io/synapse/latest/admin_api/rooms.html#list-room-api
         """
         while _from is not None:
-            response = self.query("get", "v1/rooms", params={
+            params = {
                 "from": _from,
                 "limit": limit,
                 "search_term": name,
                 "order_by": order_by,
                 "dir": "b" if reverse else None
-            })
+            }
+            if empty_rooms is not None:
+                params["empty_rooms"] = str(empty_rooms).lower()
+            response = self.query("get", "v1/rooms",
+                                  params=params)
             yield response
             _from = response.get("next_batch", None)
             self.log.debug(f"room_list_paginate: next from value = {_from}")
@@ -1035,7 +1304,7 @@ class SynapseAdmin(ApiRequest):
                                      _size_gt, delete_profiles):
         """ Delete local media by date and/or size FIXME and/or?
         """
-        if before_days:
+        if before_days or before_days == 0:
             self.log.debug("Received --before-days: %s", before_days)
             before_ts = self._timestamp_from_days_ago(before_days)
         elif before:
@@ -1085,7 +1354,8 @@ class SynapseAdmin(ApiRequest):
     def purge_media_cache(self, before_days, before, _before_ts):
         """ Purge old cached remote media
         """
-        if before_days:
+        # TODO: handle click CLI stuff in CLI, not API
+        if before_days or before_days == 0:
             self.log.debug("Received --before-days: %s", before_days)
             before_ts = self._timestamp_from_days_ago(before_days)
         if before:
@@ -1123,7 +1393,7 @@ class SynapseAdmin(ApiRequest):
         """ Purge room history
         """
         before_ts = None
-        if before_days:
+        if before_days or before_days == 0:
             self.log.debug("Received --before-days: %s", before_days)
             before_ts = self._timestamp_from_days_ago(before_days)
         elif before:
@@ -1327,7 +1597,7 @@ class SynapseAdmin(ApiRequest):
         """
         # t because query also accepts token when we want it for the
         # request
-        # https://github.com/JOJ0/synadm/issues/110#issuecomment-1590032158
+        # https://codeberg.org/synadm/synadm/issues/110#issuecomment-3112443
         return self.query("delete", "v1/registration_tokens/{t}",
                           t=token)
 
