@@ -1,6 +1,7 @@
 # -*- coding: utf-8 -*-
 # synadm
-# Copyright (C) 2020-2023 Johannes Tiefenbacher
+# Copyright (C) 2021-2025 Johannes Tiefenbacher
+# Copyright (C) 2023-2025 JacksonChen666
 #
 # synadm is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,6 +23,7 @@ import click
 from click_option_group import optgroup
 from click_option_group import RequiredAnyOptionGroup, OptionGroup
 from click_option_group import RequiredMutuallyExclusiveOptionGroup
+import urllib.parse
 
 from synadm import cli
 
@@ -96,7 +98,7 @@ def media_list_cmd(ctx, helper, room_id, user_id, from_, limit, sort, reverse,
 @media.command(name="quarantine")
 @optgroup.group(
     "Quarantine media by",
-    cls=RequiredAnyOptionGroup,
+    cls=RequiredMutuallyExclusiveOptionGroup,
     help="")
 @optgroup.option(
     "--media-id", "-i", type=str,
@@ -110,20 +112,37 @@ def media_list_cmd(ctx, helper, room_id, user_id, from_, limit, sort, reverse,
     "--user-id", "-u", type=str,
     help="""All media uploaded by user with this matrix ID (@user:server) will
     be quarantined.""")
+@optgroup.option(
+    "--mxc-uri", "-U", type=str,
+    help="""The MXC URI of the media to quarantine. Equivalent to passing
+    media ID and server name if MXC URI is passed.""")
 @click.option(
     "--server-name", "-s", type=str,
-    help="""The server name of the media, mandatory when --media-id is used and
-    _remote_ media should be processed. For locally stored media this option
-    can be omitted.
+    help="""The server name of the media for quarantining remote media. If
+    not used, quarantines local homeserver by specified --media-id.
     """)
 @click.pass_obj
-def media_quarantine_cmd(helper, server_name, media_id, user_id, room_id):
+def media_quarantine_cmd(helper, server_name, media_id, user_id, room_id,
+                         mxc_uri):
     """ Quarantine media in rooms, by users or by media ID.
     """
+    if mxc_uri:
+        # while an MXC URI is a URI and not a URL, this works anyways for
+        # splitting things up
+        uri_parsed = urllib.parse.urlparse(mxc_uri)
+        if uri_parsed.scheme == "mxc":
+            # ........................ rm first /
+            media_id = uri_parsed.path.replace("/", "", 1)
+            server_name = uri_parsed.netloc
+        else:
+            click.echo("Passed MXC URI does not have mxc in scheme.", err=True)
+            raise SystemExit(1)
+
+    media_quarantined = None
+
     if media_id and not server_name:
         # We assume it is local media and fetch our own server name.
-        fetched_name = helper.retrieve_homeserver_name(
-            helper.config["base_url"])
+        fetched_name = helper.retrieve_homeserver_name()
         media_quarantined = helper.api.media_quarantine(fetched_name, media_id)
     elif server_name and not media_id:
         click.echo("Media ID missing.")
@@ -152,31 +171,46 @@ def media_quarantine_cmd(helper, server_name, media_id, user_id, room_id):
     help="""The media with this specific media ID will be removed from
     quarantine.
     """)
+@optgroup.option(
+    "--mxc-uri", "-U", type=str,
+    help="""The MXC URI of the media to quarantine. Equivalent to passing
+    media ID and server name if MXC URI is passed.""")
 @click.option(
     "--server-name", "-s", type=str,
-    help="""The server name of the media, mandatory when --media-id is used and
-    _remote_ media should be processed. For locally stored media this option
-    can be omitted.
-    """)
+    help="""The server name of the media for unquarantining remote media. If
+    not used, unquarantines local homeserver by specified --media-id.""")
 @click.pass_obj
-def media_unquarantine_cmd(helper, server_name, media_id):
+def media_unquarantine_cmd(helper, server_name, media_id, mxc_uri):
     """ Remove media from quarantine.
     """
+    if mxc_uri:
+        # while an MXC URI is a URI and not a URL, this works anyways for
+        # splitting things up
+        uri_parsed = urllib.parse.urlparse(mxc_uri)
+        if uri_parsed.scheme == "mxc":
+            # ........................ rm first /
+            media_id = uri_parsed.path.replace("/", "", 1)
+            server_name = uri_parsed.netloc
+        else:
+            click.echo("Passed MXC URI does not have mxc in scheme.", err=True)
+            raise SystemExit(1)
+
+    unquarantined = None
+
     if media_id and not server_name:
         # We assume it is local media and fetch our own server name.
-        fetched_name = helper.retrieve_homeserver_name(
-            helper.config["base_url"])
-        unquarantinend = helper.api.media_unquarantine(fetched_name, media_id)
+        fetched_name = helper.retrieve_homeserver_name()
+        unquarantined = helper.api.media_unquarantine(fetched_name, media_id)
     elif server_name and not media_id:
         click.echo("Media ID missing.")
-        unquarantinend = None
+        unquarantined = None
     elif media_id and server_name:
-        unquarantinend = helper.api.media_unquarantine(server_name, media_id)
+        unquarantined = helper.api.media_unquarantine(server_name, media_id)
 
-    if unquarantinend is None:
+    if unquarantined is None:
         click.echo("Media could not be removed from quarantine.")
         raise SystemExit(1)
-    helper.output(unquarantinend)
+    helper.output(unquarantined)
 
 
 @media.command(name="protect")
@@ -215,7 +249,8 @@ def media_protect_cmd(helper, media_id):
 def media_purge_cmd(helper, before_days, before, before_ts):
     """ Purge old cached remote media
 
-    To delete local media, use `synadm media delete`
+    To delete local media, use `synadm media delete`. Specific media IDs are
+    not supported by Synapse.
     """
     media_purged = helper.api.purge_media_cache(before_days, before, before_ts)
     if media_purged is None:
@@ -274,10 +309,31 @@ def media_delete_cmd(helper, media_id, before_days, before, before_ts,
         click.echo("Combination of --media-id and --size not valid.")
         media_deleted = None
     elif media_id:
-        server_name = helper.retrieve_homeserver_name(
-                helper.config["base_url"])
+        server_name = helper.retrieve_homeserver_name()
+
+        sure = (
+            helper.no_confirm or
+            click.prompt(f"Delete LOCAL media ID {media_id}? This is an "
+                         "irreversible destructive operation! (y/N)",
+                         type=bool, default=False, show_default=False)
+        )
+        if not sure:
+            raise SystemExit(1)
+
+        # TODO: refactor CLI logic out of API
         media_deleted = helper.api.media_delete(server_name, media_id)
     else:
+        sure = (
+            helper.no_confirm or
+            # TODO: state what's being deleted
+            click.prompt("Delete various LOCAL media? This is an "
+                         "irreversible destructive operation! (y/N)",
+                         type=bool, default=False, show_default=False)
+        )
+        if not sure:
+            raise SystemExit(1)
+
+        # TODO: refactor CLI logic out of API
         media_deleted = helper.api.media_delete_by_date_or_size(
             before_days, before, before_ts, size, delete_profiles
         )
